/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.editors.panels;

import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import com.unitesk.requality.nodetypes.Requirement;

public class RequirementAdditionalPropertiesPanel extends AbstractPanel {
    static private final int HEIGHT_MULT = 100;
    static private final String EDIT_PRE = "Predicate:";
    static private final String EDIT_ATTRS = "Attributes:";
    private Text preText = null;

    private Requirement req;
    private AttributesPanel attribPanel;

    @Override
    protected void initModifyListener() {
        attribPanel.addListener( SWT.Modify, modifyListener );
        preText.addListener( SWT.Modify, modifyListener );
    }

    public void setRequirement(Requirement req) {
        this.req = req;
        if(isDisposed()) {
            return;
        }
        preText.setText(req.getPredicate());
        attribPanel.setAttributes(req);
    }

    public RequirementAdditionalPropertiesPanel(Composite parent, int style) {
        super(parent, style);

        setLayout(new GridLayout());

        GridData layoutData = new GridData(GridData.FILL_HORIZONTAL);
        layoutData.heightHint = HEIGHT_MULT;
        layoutData.horizontalSpan = 2;

        // Атрибуты
        new Label(this, SWT.NONE).setText(EDIT_ATTRS);
        attribPanel= new AttributesPanel(this, SWT.NONE);
        attribPanel.setLayoutData(layoutData);

        // Предикат
        new Label(this, SWT.NONE).setText(EDIT_PRE);
        preText = new Text(this, SWT.BORDER);
        GridData layoutDataPred = new GridData(GridData.FILL_HORIZONTAL);
        layoutData.horizontalSpan = 2;
        preText.setLayoutData(layoutDataPred);
        
        initModifyListener();
    }

    @Override
    public boolean isChanged() {
        int ignoreKeysCount=0;
        for(String key: req.getAttributeKeys()) {
            if(key.startsWith("_")) {
                ignoreKeysCount++;
            }
        }
        if(req.getAttributeKeys().size()!=attribPanel.getAttributes().keySet().size()+ignoreKeysCount) {
            return true;
        }

        for(String key : attribPanel.getAttributes().keySet()) {
            if(key.startsWith("_")) {
                continue;
            }

            if(req.getAttribute(key)==null) {
                return true;
            }
            if(!req.getAttributeValue(key).toString().equals(attribPanel.getAttributes().get(key))) {
                return true;
            }
        }
        if (!req.getPredicate().equals(preText.getText()))
            return true;
        return false;
    }

    public String getPredicate() {
        return preText.getText();
    }

    public Map<String, String> getAttributes() {
        return attribPanel.getAttributes();
    }

}
