package com.unitesk.requality.eclipse.views;

import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.navigator.CommonActionProvider;
import org.eclipse.ui.navigator.ICommonActionConstants;
import org.eclipse.ui.navigator.ICommonActionExtensionSite;
import org.eclipse.ui.navigator.ICommonMenuConstants;
import org.eclipse.ui.navigator.ICommonViewerSite;

import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.DocumentProcessorDescriptor;
import com.unitesk.requality.eclipse.views.actions.CNFAction;
import com.unitesk.requality.eclipse.views.actions.GenerateReportAction;
import com.unitesk.requality.eclipse.views.actions.OpenAction;
import com.unitesk.requality.eclipse.views.actions.documents.CreateDocumentFolderAction;
import com.unitesk.requality.eclipse.views.actions.documents.DocumentProcessorAction;
import com.unitesk.requality.eclipse.views.actions.documents.ImportAction;
import com.unitesk.requality.eclipse.views.actions.documents.RemoveDocumentAction;
import com.unitesk.requality.eclipse.views.actions.requirements.AddRequirementAction;

public class CNFActionProvider extends CommonActionProvider {

    private ICommonViewerSite site;

    public CNFActionProvider() {
    }

    @Override
    public void init(ICommonActionExtensionSite aSite) {
        site = aSite.getViewSite();
    }

    @Override
    public void fillContextMenu(IMenuManager manager) {
        super.fillContextMenu(manager);
        ISelectionProvider provider = site.getSelectionProvider();
        CNFAction addAction = new AddRequirementAction(provider);
        if (addAction.isEnabled()) {
            manager.appendToGroup(ICommonMenuConstants.GROUP_NEW, addAction);
        }

        GenerateReportAction generateReportAction = new GenerateReportAction(
                provider);
        if (generateReportAction.isEnabled()) {
            manager.appendToGroup(ICommonMenuConstants.GROUP_GENERATE,
                    generateReportAction);
        }

        ImportAction docImport = new ImportAction(provider);
        if(docImport.isEnabled()) {
            manager.appendToGroup(ICommonMenuConstants.GROUP_NEW, docImport);
        }

        CreateDocumentFolderAction createDocFolderAction = new CreateDocumentFolderAction(
                provider);
        if (createDocFolderAction.isEnabled()) {
            manager.appendToGroup(ICommonMenuConstants.GROUP_NEW,
                    createDocFolderAction);
        }

        RemoveDocumentAction docRemove = new RemoveDocumentAction(provider);
        if (docRemove.isEnabled()) {
            manager.appendToGroup(ICommonMenuConstants.GROUP_EDIT, docRemove);
        }

        for (DocumentProcessorDescriptor procDesc : Activator.getDefault()
                .getProcessors()) {
            DocumentProcessorAction docProc = new DocumentProcessorAction(
                    procDesc, provider);
            if (docProc.isEnabled()) {
                manager.appendToGroup(ICommonMenuConstants.GROUP_GENERATE,
                        docProc);
            }
        }
    }

    @Override
    public void fillActionBars(IActionBars actionBars) {
        super.fillActionBars(actionBars);
        actionBars.setGlobalActionHandler(ICommonActionConstants.OPEN,
                new OpenAction((IStructuredSelection) site
                        .getSelectionProvider().getSelection()));
    }
}
