package com.unitesk.requality.eclipse.views;

import java.util.UUID;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.model.WorkbenchLabelProvider;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.tools.ResourceStorage;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.nodetypes.Requirement;

public class CNFLabelProvider extends LabelProvider implements ILabelProvider {

    @Override
    public Image getImage(Object element) {
        if (element instanceof IFile) {
            TreeDB db = TreesTracker.getTracker().getTree(
                    ((IFile) element).getProject());
            ResourceStorage rs = (ResourceStorage) db.getStorage();
            IFile f = (IFile) element;
            if (!f.exists()) {
                return PlatformUI.getWorkbench().getSharedImages()
                        .getImage(ISharedImages.IMG_DEC_FIELD_ERROR);
            }
            UUID uuid = rs.getUUId((IFile) element);
            if (uuid != null) {
                return getImage(db.getNode(uuid));
            }
        }

        if (element instanceof IProject) {
            TreeDB db = TreesTracker.getTracker().getTree(((IProject) element));
            if (db != null) {
                return PlatformUI.getWorkbench().getSharedImages()
                        .getImage(IDE.SharedImages.IMG_OBJ_PROJECT);
            }
        }

        if (element instanceof TreeDB) {
            return PlatformUI.getWorkbench().getSharedImages()
                    .getImage(IDE.SharedImages.IMG_OBJ_PROJECT);
        }

        if (element instanceof TreeNode) {
            TreeNode node = (TreeNode) element;
            if (node.getQualifiedId().equals("/")) {
                return PlatformUI.getWorkbench().getSharedImages()
                        .getImage(IDE.SharedImages.IMG_OBJ_PROJECT);
            }
            return Activator.getDefault().getImageRegistry()
                    .get("nodetype." + node.getType());
        }
        return WorkbenchLabelProvider.getDecoratingWorkbenchLabelProvider().getImage(element);
    }

    @Override
    public String getText(Object element) {
        if (element instanceof IFile) {
            TreeDB db = TreesTracker.getTracker().getTree(
                    ((IFile) element).getProject());
            ResourceStorage rs = (ResourceStorage) db.getStorage();
            IFile f = (IFile) element;
            if (!f.exists()) {
                return "n/a";
            }
            UUID uuid = rs.getUUId(f);
            if (uuid != null) {
                return getText(db.getNode(uuid));
            }
        }

        if (element instanceof IProject) {
            TreeDB db = TreesTracker.getTracker().getTree(((IProject) element));
            if (db != null) {
                return ((IProject) element).getName();
            }
        }

        if (element instanceof TreeDB) {
            return TreesTracker.getTracker().getProject((TreeDB) element).getName();
        }
        if (element instanceof Requirement) {
            Requirement node = (Requirement) element;
            return node.getName();
        }
        if (element instanceof TreeNode) {
            TreeNode node = (TreeNode) element;
            if (node.getQualifiedId().equals("/")) {
                return TreesTracker.getTracker().getProject(node.getTreeDB())
                        .getName();
            }
            return node.getId();
        }
        return WorkbenchLabelProvider.getDecoratingWorkbenchLabelProvider().getText(element);
    }

}
