/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.views.actions;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.NodeTypeDescriptor;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.nodetypes.Requirement;

public class OpenAction extends Action {
    private IStructuredSelection selection;

    // TODO: bind editor with nodetype
    // private final static String[] supportedTypes = new String[] { "Document",
    // "Requirement" };

    public OpenAction(IStructuredSelection selection) {
        setText("Open requirement");
        this.selection = selection;
    }

    @Override
    public boolean isEnabled() {
        TreeNode node = getSelectedNode();
        if(node==null) return false;
        NodeTypeDescriptor desc = Activator.getAdditionalTypes().get(
                node.getType());
        if (desc != null && desc.getEditorId() != null
                && desc.getEditorId().length() > 0 && !node.getQualifiedId().equals(Requirement.getTypeRootQId())) {
            return true;
        }
        return false;
    }

    public void run() {
        try {
            runAction();
        } catch (PartInitException e) {
            e.printStackTrace();
        }
    }

    protected TreeNode getSelectedNode() {
        ITreeSelection selected = (ITreeSelection) selection;
        if (selected.isEmpty()) {
            return null;
        }

        Object obj = selected.getFirstElement();
        if (obj instanceof TreeNode) {
            return (TreeNode) obj;
        }

        return null;
    }

    private void runAction() throws PartInitException {
        if (!isEnabled()) {
            return;
        }
        TreeNode node = getSelectedNode();
        NodeTypeDescriptor desc = Activator.getAdditionalTypes().get(
                node.getType());
        IWorkbenchPage page = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getActivePage();
        if (page == null)
            return;
        NodeEditorInput input = new NodeEditorInput(node);
        page.openEditor(input, desc.getEditorId(), true);
    }
}
