/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.actions.requirements;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.UUID;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.editors.browser.ReqMarker;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

public class RemoveRequirementAction implements IObjectActionDelegate {
    private TreeNode nodes[];
    public RemoveRequirementAction() {
    }

    @Override
    public void run(IAction action) {
        if(nodes.length > 0) {
            if (!MessageDialog.openConfirm(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), "Delete requirement",
                    "Delete selected requirement")) {
                return;
            }
            for(TreeNode node : nodes) {
                deleteLocations((Requirement) node);
                node.delete();
            }
        }
    }

    // TODO: (low) check for crosslinks to locations
    private void deleteLocations(Requirement req) {
        for (Location loc : req.getLocations()) {
            if (loc == null) {
                // FIXME:
                continue;
            }
            try {
                Document doc = (Document) loc.getParent();
                if (doc == null)
                    return;
                IEditorReference eRef = Activator.findMarkEditorForDocument(doc
                        .getQualifiedId());
                if (eRef != null) {
                    if (eRef.getEditor(true) != null
                            && (eRef.getEditor(true) instanceof ReqMarker)) {
                        ReqMarker marker = (ReqMarker) eRef.getEditor(true);
                        marker.getBrowser().deleteLocation(
                                UUID.fromString(loc.getId()));
                        marker.saveDocument();
                    }
                } else {
                    doc.removeLocation(loc.getId());
                }
                doc.removeChild(loc);
                TreesTracker.getResourceByNode(doc).refreshLocal(IResource.DEPTH_INFINITE, null);
            } catch (PartInitException e) {
                e.printStackTrace();
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            } catch (CoreException e) {
                e.printStackTrace();
            }
        }
        for (int i = 0; i < req.sizeChildren(); i++) {
            TreeNode node = req.getChild(i);
            if (node instanceof Requirement) {
                Requirement r = (Requirement) node;
                deleteLocations(r);
            }
        }
    }

    @Override
    public void selectionChanged(IAction action, ISelection selection) {
        nodes = getNodesToDelete(selection);
        if(nodes.length == 0) {
            action.setEnabled(false);
            return;
        }
        action.setEnabled(true);
        if(nodes.length == 1) {
            action.setText("Remove "+nodes[0].getType());
        }
        
        if(nodes.length > 1) {
            action.setText("Remove nodes (" + nodes.length + ")");
        }
        
    }

    private TreeNode[] getNodesToDelete(ISelection selection) {
        ArrayList<TreeNode> result = new ArrayList<TreeNode>();
        if (selection instanceof IStructuredSelection) {
            IStructuredSelection ssel = (IStructuredSelection) selection;
            for (Object o : ssel.toList()) {
                if (o instanceof Requirement) {
                    TreeNode node = (TreeNode) o;
                    result.add(node);
                }
            }
        }
        return result.toArray(new TreeNode[0]);
    }

    @Override
    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
    }
}
