/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package com.unitesk.requality.eclipse.wizards;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.views.TreesContentProvider;
import com.unitesk.requality.eclipse.views.actions.documents.ImportAction;
import com.unitesk.requality.eclipse.views.documents.DocLabelProvider;
import com.unitesk.requality.nodetypes.DocFolder;
import com.unitesk.requality.nodetypes.Document;

public class DocumentImportWizardPage extends WizardPage {

    private TreeViewer docFolder;
    private Text fileText;
    private TreeDB selectedDB;

    private static String[] exts = { "*.html; *.htm; *.xhtml" };
    private static String[] extNames = { "Markup files (*.html; *.htm; *.xhtml)" };

    private Listener changeListener = new Listener() {
        @Override
        public void handleEvent(Event event) {
            DocumentImportWizardPage.this.getContainer().updateButtons();
        }
    };

    private SelectionListener butList = new SelectionListener() {
        public void widgetDefaultSelected(SelectionEvent e) {
        }

        public void widgetSelected(SelectionEvent e) {
            FileDialog fileSelect = new FileDialog(getShell());
            if(ImportAction.initalPath!=null) {
                fileSelect.setFilterPath(ImportAction.initalPath);
            }
            fileSelect.setFileName(fileText.getText());
            fileSelect.setFilterExtensions(exts);
            fileSelect.setFilterNames(extNames);
            String path = fileSelect.open();
            if (path != null)
                fileText.setText(path);
        }
    };

    protected DocumentImportWizardPage() {
        super("Document import");
    }

    @Override
    public void createControl(Composite parent) {
        Composite panel = new Composite(parent, SWT.NONE);
        panel.setLayout(new GridLayout(1, false));

        new Label(panel, SWT.NONE).setText("Select HTML document to import: ");
        Composite selectFilePanel = new Composite(panel, SWT.NONE);
        selectFilePanel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        selectFilePanel.setLayout(new GridLayout(2, false));

        fileText = new Text(selectFilePanel, SWT.BORDER);
        fileText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        fileText.addListener(SWT.Modify, changeListener);

        Button selectFile = new Button(selectFilePanel, SWT.NONE);
        selectFile.setText("Browse...");
        selectFile.addSelectionListener(butList);

        new Label(panel, SWT.NONE).setText("Select file location in project: ");
        docFolder = new TreeViewer(panel);
        GridData dfLayoutData = new GridData(GridData.FILL_BOTH);
        dfLayoutData.horizontalSpan = 2;
        docFolder.getControl().setLayoutData(dfLayoutData);

        docFolder.setContentProvider(new TreesContentProvider(Document.getTypeRootQId()) {
            @Override
            public Object[] getChildren(Object parentElement) {
                List<Object> normalResult = Arrays.asList(super
                        .getChildren(parentElement));
                List<Object> result = new ArrayList<Object>();
                for (Object o : normalResult) {
                    if (!(o instanceof Document)) {
                        result.add(o);
                    }
                }
                return result.toArray();
            }

            @Override
            public boolean hasChildren(Object parentElement) {
                return getChildren(parentElement).length != 0;
            }
        });

        docFolder.setLabelProvider(new DocLabelProvider());
        docFolder.setInput(ResourcesPlugin.getWorkspace().getRoot());
        docFolder.setSorter(new ViewerSorter());
        docFolder
                .addPostSelectionChangedListener(new ISelectionChangedListener() {
                    @Override
                    public void selectionChanged(SelectionChangedEvent event) {
                        changeListener.handleEvent(new Event());
                    }
                });
        docFolder.getControl().addListener(SWT.Modify, changeListener);

        setControl(parent);
    }

    public TreeNode getDocumentParent() {
        StructuredSelection selection = (StructuredSelection) docFolder
                .getSelection();
        if (selection.isEmpty())
            return null;

        if (selection.getFirstElement() instanceof DocFolder) {
            DocFolder docFolder = (DocFolder) selection.getFirstElement();
            selectedDB = docFolder.getTreeDB();
            return docFolder;
        }

        if (selection.getFirstElement() instanceof TreeDB) {
            TreeDB treeDB = (TreeDB) selection.getFirstElement();
            selectedDB = treeDB;
            return treeDB.getNode(Document.getTypeRootQId());
        }
        return null;
    }

    public TreeDB getSelectedDB() {
        return selectedDB;
    }

    public String getImportFileName() {
        return fileText.getText();
    }

    public boolean isDataReady() {
        if (!new File(fileText.getText()).exists())
            return false;
        if (getDocumentParent() == null)
            return false;
        if (getSelectedDB() == null)
            return false;
        return true;
    }
}