package com.unitesk.requality.merge;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.NamedNodeMap;

public class Location {

	private Node spanNode;
	private String anchorText;
	private String locationText;
	private Node anchorNode;
	private Node leftSibling;
	private Node rightSibling;
	private String id;

	public Location(Node x) throws InvalidLocationException {
		if (!isLocationNode(x))
			throw new InvalidLocationException();
		this.spanNode = x;
		id = retrieveLocationId(x);
		anchorText = retrieveAnchorText(x);
	}

	private String retrieveLocationId(Node x) {
		org.w3c.dom.Node atr = x.getDomNode().getAttributes()
				.getNamedItem("class");
		String atrValue = atr.getNodeValue();
		String id = atrValue.substring(atrValue.indexOf("id_") + 3);
		return id;
	}

	private String retrieveAnchorText(Node x) throws InvalidLocationException {
		List<Node> spanChilds = x.getChilds();
		int textIndex = 0;
		for (Node child : spanChilds)
			if (child.getDomNode().getNodeType() == org.w3c.dom.Node.TEXT_NODE)
				textIndex = spanChilds.indexOf(child);
		if (!spanChilds.get(textIndex).isLeaf())
			throw new InvalidLocationException();
		if (spanChilds.get(textIndex).getDomNode().getNodeType() != org.w3c.dom.Node.TEXT_NODE)
			throw new InvalidLocationException();
		locationText = spanChilds.get(textIndex).getDomNode()
				.getNodeValue();
		anchorNode = spanChilds.get(textIndex);
		String anchorTextPrefix = "";
		String anchorTextPostfix = "";
		List<Node> siblings = x.getParent().getChilds();
		if (siblings.size() != 1) {
			if (x.getContext().getPosition() > 1) {
				Node leftSibling = siblings
						.get(x.getContext().getPosition() - 2);
				if (leftSibling.getDomNode().getNodeType() == org.w3c.dom.Node.TEXT_NODE) {
					String nodeValue = leftSibling.getDomNode().getNodeValue();
					if (!nodeValue.trim().isEmpty())
						anchorTextPrefix = leftSibling.getDomNode()
								.getNodeValue();
					this.leftSibling = leftSibling;
				}
			}
			if (x.getContext().getPosition() < siblings.size()) {
				Node rightSibling = siblings.get(x.getContext().getPosition());
				if (rightSibling.getDomNode().getNodeType() == org.w3c.dom.Node.TEXT_NODE) {
					String nodeValue = rightSibling.getDomNode().getNodeValue();
					if (!nodeValue.trim().isEmpty())
						anchorTextPostfix = rightSibling.getDomNode()
								.getNodeValue();
					this.rightSibling = rightSibling;
				}
			}
		}
		return (anchorTextPrefix + locationText + anchorTextPostfix);
	}

	public String getLocationId() {
		return id;
	}

	public String getAnchorText() {
		return anchorText;
	}

	public Node getSpanNode() {
		return spanNode;
	}

	public Node getLeftSibling() {
		return leftSibling;
	}

	public Node getRightSibling() {
		return rightSibling;
	}
	
	public List<Node> getRelatedNodes() {
	    List<Node> nodes = new ArrayList<Node>();
	    if (leftSibling != null)
		nodes.add(leftSibling);
	    nodes.add(spanNode);
	    if (rightSibling != null)
		nodes.add(rightSibling);
	    return nodes;
	}
	
	public static boolean isLocationNode(Node n) {
		org.w3c.dom.Node x = n.getDomNode();
		NamedNodeMap attrs = x.getAttributes();
		if (attrs == null)
			return false;
		org.w3c.dom.Node classAtt = attrs.getNamedItem("class");
		if (classAtt == null)
			return false;
		String attValue = classAtt.getNodeValue();
		if (!attValue.contains("requality_text"))
			return false;
		return true;
	}

	public String toString() {
	    return this.id+" : "+this.anchorText;
	}
	
	public Node getAnchorNode() {
	    return anchorNode;
	}
	
	public void setAnchorNode(Node newAnchorNode) {
	    this.anchorNode = newAnchorNode;
	}
	
	@SuppressWarnings("serial")
	public class InvalidLocationException extends Exception {

	}

	public String getLocationText() {
	    return this.locationText;
	}
	
	public boolean wasUpdated() {
	    if(this.anchorNode==null || !this.spanNode.getDomNode().getOwnerDocument().isSameNode(this.anchorNode.getDomNode().getOwnerDocument()))
		return true;
	    else
		return false;
	}
}
