package com.unitesk.requality.nodetypes;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Result;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.documents.DeepFirstXMLDOMWalker;

public class Document extends TreeNode {
    private static final String ATTRIBUTE_FILE = "_file";
    private static final String TYPE_ROOT_QID = "/Documents";

    public Document() {
        super(null, null);
    }

    private Element getElementChildIgnoreCase(Node node, String name) {
        for (int i = 0; i < node.getChildNodes().getLength(); i++) {
            if (node.getChildNodes().item(i).getNodeName().toLowerCase()
                    .equals(name.toLowerCase())) {
                return (Element) node.getChildNodes().item(i);
            }
        }
        return null;
    }

    public void setMetaCharset(String charset) {
        try {
            org.w3c.dom.Document dom = getDOM();
            Node html = dom.getFirstChild();
            Node head = getElementChildIgnoreCase(dom.getFirstChild(), "head");
            if (head == null) {
                head = dom.createElement("head");
                html.appendChild(head);
            }
            Element meta = getElementChildIgnoreCase(head, "meta");
            if (meta == null) {
                meta = dom.createElement("meta");
                head.appendChild(meta);
            }
            meta.setAttribute("http-equiv", "Content-Type");
            meta.setAttribute("content", "text/html; charset=" + charset);
            saveDOM();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public String getMetaCharset() {
        // TODO: check rfc for this meta
        String result = null;
        try {
            DeepFirstXMLDOMWalker dfw = new DeepFirstXMLDOMWalker(getDOM()
                    .getFirstChild());
            for (Node node : dfw) {
                if (node.getNodeName().toLowerCase().equals("meta")) {
                    if (node.getAttributes().getNamedItem("http-equiv") == null)
                        continue;
                    if (!node.getAttributes().getNamedItem("http-equiv")
                            .getNodeValue().toLowerCase()
                            .equals("content-type"))
                        continue;
                    if (node.getAttributes().getNamedItem("content") == null)
                        continue;
                    String content = node.getAttributes()
                            .getNamedItem("content").getNodeValue();
                    int i = content.indexOf("charset=");
                    if (i < 0)
                        return null;
                    return content.substring(i + "charset=".length());
                }
            }
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return result;
    }

    public static String getTypeRootQId() {
        return TYPE_ROOT_QID;
    }

    public Document(TreeDB db, UUID uuid) {
        super(db, uuid);
        setType("Document");
    }

    public void setResourceName(String name) {
        Attribute a = new Attribute(AttributeType.RESOURCE, ATTRIBUTE_FILE,
                name);
        putAttribute(a);
    }

    public String getResourceName() {
        return (String) getAttributeValue(ATTRIBUTE_FILE);
    }

    private static final String LOCATION_CLASS_ID_PREFIX = "requality_text id_";
    private org.w3c.dom.Document xmlDoc = null;

    public String getHTMLPath() {
        return getTreeDB().getStorage().getNodeResourcePath(getUUId())
                + File.separator + getResourceName();
    }

    public List<UUID> getLocations() {
        List<UUID> result = new ArrayList<UUID>();
        try {
            DeepFirstXMLDOMWalker dfw = new DeepFirstXMLDOMWalker(getDOM()
                    .getFirstChild());
            for (Node node : dfw) {
                if (node.getNodeName().equals("span")) {
                    Node spanClass = node.getAttributes().getNamedItem("class");
                    if (spanClass != null) {
                        String className = spanClass.getNodeValue();
                        if (className.startsWith(LOCATION_CLASS_ID_PREFIX)) {
                            UUID uuid = UUID.fromString(className
                                    .substring(LOCATION_CLASS_ID_PREFIX
                                            .length()));
                            if (!result.contains(uuid)) {
                                result.add(uuid);
                            }
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    public void removeLocation(String uuid) {
        try {
            removeLocation(UUID.fromString(uuid), getDOM().getFirstChild());
            saveDOM();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void saveDOM() {
        String path = getHTMLPath();
        try {
            org.w3c.dom.Document doc = getDOM();
            Writer out = new OutputStreamWriter(new FileOutputStream(path),
                    "UTF-8");
            Result result = new StreamResult(out);
            TransformerFactory.newInstance().newTransformer()
                    .transform(new DOMSource(doc), result);
            out.close();

        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    // TODO: nested location support
    private void removeLocation(UUID uuid, Node node) {
        if (isLocationNode(node)) {
            if (getUUID(node).equals(uuid)) {
                // delete node
                for (int i = 0; i < node.getChildNodes().getLength(); i++) {
                    Node curNode = node.getChildNodes().item(i);
                    if (curNode.getNodeName().equals("a")) {
                        if (curNode.getAttributes().getNamedItem("id")
                                .getNodeValue().equals("id_" + uuid)) {
                            continue;
                        }
                    }
                    if (curNode.getNodeName().equals("span")) {
                        if (curNode
                                .getAttributes()
                                .getNamedItem("class")
                                .getNodeValue()
                                .startsWith(
                                        "requality_link requality_link_" + uuid)) {
                            continue;
                        }
                    }
                    node.getParentNode().appendChild(curNode);
                }
                node.getParentNode().removeChild(node);
            }
        }

        for (int i = 0; i < node.getChildNodes().getLength(); i++) {
            removeLocation(uuid, node.getChildNodes().item(i));
        }
    }

    private boolean isLocationNode(Node node) {
        if (node.getNodeName().equals("span")) {
            Node spanClass = node.getAttributes().getNamedItem("class");
            if (spanClass == null) {
                return false;
            }

            String className = spanClass.getNodeValue();
            if (className.startsWith(LOCATION_CLASS_ID_PREFIX)) {
                return true;
            }
        }
        return false;
    }

    private UUID getUUID(Node locationNode) {
        try {
            String className = locationNode.getAttributes()
                    .getNamedItem("class").getNodeValue();
            if (className.startsWith(LOCATION_CLASS_ID_PREFIX)) {
                return UUID.fromString(className
                        .substring(LOCATION_CLASS_ID_PREFIX.length()));
            }
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
        return null;
    }

    public Location getLocation(String locationId) {
        return (Location) findChild(locationId);
    }

    public org.w3c.dom.Document getDOM() throws Exception {
        if (xmlDoc == null) {
            DocumentBuilder docBuilder = DocumentBuilderFactory.newInstance()
                    .newDocumentBuilder();
            xmlDoc = docBuilder.parse(getHTMLPath());
        }
        return xmlDoc;
    }

    public int sizeLocations() {
        return getLocations().size();
    }
}
