package com.unitesk.requality.nodetypes;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.UUID;
import java.util.regex.Pattern;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Node;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.tools.DeepFirstTreeWalker;

public class Location extends TreeNode {
    public Location() {
        super(null, null);
    }

    public Location(TreeDB db, UUID uid) {
        super(db, uid);
        setType("Location");
    }

    public String getDescription(String basePath) {
        String result = getDescription("<div>", "</div>", basePath);
        // Remove <div> tags
//        if(result.startsWith("<div>") && result.endsWith("</div>")) {
//            result = result.substring(5, result.length() - 6);
//        }
        return result;
    }

    public Requirement[] getRequirements() {
        ArrayList<Requirement> result = new ArrayList<Requirement>();
        for (TreeNode node : new DeepFirstTreeWalker(getTreeDB().getNode(
                Requirement.getTypeRootQId()))) {
            if (!node.getType().equals("Requirement")) {
                continue;
            }
            Requirement req = (Requirement) node;
            if (req.getLocations().contains(this))
                result.add(req);
        }
        return result.toArray(new Requirement[0]);
    }

    public String getDescription(String prefix, String postfix, String basePath) {
        ByteArrayOutputStream result = new ByteArrayOutputStream();
        try {
            DocumentBuilder docBuilder = DocumentBuilderFactory.newInstance()
                    .newDocumentBuilder();
            String content = prefix + super.getDescription() + postfix;

            Document doc = docBuilder.parse(new ByteArrayInputStream(content
                    .getBytes("UTF-8")));
            String htmlFolderPath = new File(
                    ((com.unitesk.requality.nodetypes.Document) getParent())
                            .getHTMLPath()).getParent();
            String relPath = calcRelPath(htmlFolderPath, basePath) + "/";
            walk(doc.getFirstChild(), relPath, htmlFolderPath);
            DocumentFragment fragment = doc.createDocumentFragment();
            for(int i=0; i < doc.getFirstChild().getChildNodes().getLength(); i++) {
                fragment.appendChild(doc.getFirstChild().getChildNodes().item(i));
            }
            TransformerFactory transFactory = TransformerFactory.newInstance();
            Transformer transformer = transFactory.newTransformer();
            transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION,
                    "yes");
            transformer.transform(new DOMSource(fragment), new StreamResult(result));
            return result.toString("UTF-8");
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "error!" + result.toString();
    }

    private String calcRelPath(String htmlPath, String basePath) {
        try {
            return getRelativeFile(new File(htmlPath), new File(basePath))
                    .getPath();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    private void walk(Node node, String relPath, String htmlFolderPath) {
        proccess(node, relPath, htmlFolderPath);
        for (int i = 0; i < node.getChildNodes().getLength(); i++) {
            walk(node.getChildNodes().item(i), relPath, htmlFolderPath);
        }
    }

    private void proccess(Node node, String relPath, String htmlFolderPath) {
        String nodeName = node.getNodeName().toLowerCase();
        if (nodeName.equals("img")) {
            Node attr = node.getAttributes().getNamedItem("src");
            if (attr != null) {
                if (new File(htmlFolderPath + File.separator
                        + attr.getNodeValue()).exists()) {
                    String newPath = relPath + attr.getNodeValue();
                    attr.setNodeValue(newPath);
                }
            }
        }
    }

    @Override
    public String toString() {
        return "Location: " + getId().toString() + " [" + getDescription()
                + "] in " + getParent().getQualifiedId();
    }

    /**
     * Returns the path of one File relative to another.
     * 
     * @param target
     *            the target directory
     * @param base
     *            the base directory
     * @return target's path relative to the base directory
     * @throws IOException
     *             if an error occurs while resolving the files' canonical names
     */
    public static File getRelativeFile(File target, File base)
            throws IOException {
        String[] baseComponents = base.getCanonicalPath().split(
                Pattern.quote(File.separator));
        String[] targetComponents = target.getCanonicalPath().split(
                Pattern.quote(File.separator));
        // skip common components
        int index = 0;
        for (; index < targetComponents.length && index < baseComponents.length; ++index) {
            if (!targetComponents[index].equals(baseComponents[index]))
                break;
        }
        StringBuilder result = new StringBuilder();
        if (index != baseComponents.length) {
            // backtrack to base directory
            for (int i = index; i < baseComponents.length; ++i)
                result.append(".." + File.separator);
        }
        for (; index < targetComponents.length; ++index)
            result.append(targetComponents[index] + File.separator);
        if (!target.getPath().endsWith("/") && !target.getPath().endsWith("\\")) {
            // remove final path separator
            result.delete(result.length() - "/".length(), result.length());
        }
        return new File(result.toString());
    }
}
