package com.unitesk.requality.tests;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.TreeStorage;
import com.unitesk.requality.eclipse.natures.ReqCatalogue;
import com.unitesk.requality.eclipse.tools.ResourceStorage;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.tests.core.StorageTest;

public class ResourceStorageTest extends StorageTest {
	private IProject project;
	private ResourceStorage storage;

	@Override
	protected TreeDB getDB() {
		return TreesTracker.getTracker().getTree(project);
	}

	@Before
	public void setUp() {
		storage = (ResourceStorage) createStorage();
	}

	@Override
	protected TreeStorage createStorage() {
		if (storage != null) {
			return storage;
		}
		try {
			project = ResourcesPlugin.getWorkspace().getRoot()
					.getProject("TestProject");
			if(project.exists()) {
				project.delete(false, null);
			}
			project.create(null);
			project.open(null);

			IProjectDescription description = project.getDescription();
			String[] natures = description.getNatureIds();
			String[] newNatures = new String[natures.length + 1];
			System.arraycopy(natures, 0, newNatures, 0, natures.length);
			newNatures[natures.length] = ReqCatalogue.ID;
			description.setNatureIds(newNatures);
			project.setDescription(description, null);

			return new ResourceStorage(project);
		} catch (CoreException e) {
			e.printStackTrace();
		}
		return null;
	}

	@Test
	public void rootCreationTest() {
		TreeNode root = getDB().getRootNode();
		assertNotNull(root);
		ResourceStorage rst = (ResourceStorage) getDB().getStorage();
		assertTrue(rst.canLoad(root.getUUId()));
		assertNull(rst.getParent(root.getUUId()));
		assertTrue(project.getFile("root.json").exists());
	}

	@After
	public void tearDown() {
		try {
			project.delete(false, null);
		} catch (CoreException e) {
			e.printStackTrace();
		}
	}
}
