/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.tests.core;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.UUID;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import com.unitesk.requality.core.FSStorage;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.tests.utils.Ulils;

public class DocumentTest {
    private Document doc;

    private String xhtmlString = "<html xmlns=\"http://www.w3.org/1999/xhtml\"><head><title>Sample.html</title></head>"
            + "<body>\n\n\n<h1><span class=\"requality_text id_78885e7c-9d0c-48fa-a016-0f2801bc5017\">"
            + "<a name=\"78885e7c-9d0c-48fa-a016-0f2801bc5017\" id=\"id_78885e7c-9d0c-48fa-a016-0f2801bc5017\" class=\"requality_id\"/>"
            + "He</span>ll<span class=\"requality_text id_bb36a2bb-bae0-4922-adc7-ce3f2bde484f\">"
            + "<a name=\"bb36a2bb-bae0-4922-adc7-ce3f2bde484f\" id=\"id_bb36a2bb-bae0-4922-adc7-ce3f2bde484f\" class=\"requality_id\"/>o!"
            + "</span></h1><span class=\"requality_text id_bb36a2bb-bae0-4922-adc7-ce3f2bde484f\">\n"
            + "</span><p><span class=\"requality_text id_bb36a2bb-bae0-4922-adc7-ce3f2bde484f\">wo</span>rld</p>\n"
            + "<span class=\"requality_text id_cf89e2a1-3201-4744-90eb-32463d61cf30\">"
            + "<a name=\"cf89e2a1-3201-4744-90eb-32463d61cf30\" id=\"id_cf89e2a1-3201-4744-90eb-32463d61cf30\" class=\"requality_id\"/>Many words!</span>\n\n\n"
            + "</body></html>";

    @Before
    public void setUp() throws IOException {
        String path = System.getProperty("java.io.tmpdir") + File.separator
                + "treedb.test";
        if (new File(path).exists()) {
            Ulils.recursiveDelete(new File(path));
        }
        TreeDB db = new TreeDB(new FSStorage(path));
        db.addNodeType("Document", Document.class);
        TreeNode docRoot = db.getRootNode().createChild("Documents", "Document");
        doc = (Document) docRoot.createChild("test.xhtml");
        new File(doc.getHTMLPath()).getParentFile().mkdirs();
        new File(doc.getHTMLPath()).createNewFile();
        FileWriter fw = new FileWriter(doc.getHTMLPath());
        fw.write(xhtmlString);
        fw.close();
    }

    @Test
    public void countLocations() {
        Assert.assertEquals(3, doc.sizeLocations());
    }

    @Test
    public void changeMetaCharset() {
        Assert.assertNull(doc.getMetaCharset());
        doc.setMetaCharset("cp1251");
        doc.saveDOM();
        doc.setMetaCharset("UTF-8");
        doc.saveDOM();
        Assert.assertEquals("UTF-8", doc.getMetaCharset());
    }

    @Test
    public void removeLocation() {
        UUID uuid = doc.getLocations().get(0);
        UUID[] outherUUId = new UUID[] { doc.getLocations().get(1),
                doc.getLocations().get(2) };
        Assert.assertEquals(3, doc.sizeLocations());
        doc.removeLocation(uuid.toString());
        Assert.assertEquals(2, doc.sizeLocations());
        Assert.assertEquals(outherUUId[0], doc.getLocations().get(0));
        Assert.assertEquals(outherUUId[1], doc.getLocations().get(1));
    }
}
