package com.unitesk.requality.tests.merge.sanity;

import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;

import com.unitesk.requality.merge.Location;
import com.unitesk.requality.merge.MergeProcessor;
import com.unitesk.requality.merge.Node;
import com.unitesk.requality.merge.Tree;
import com.unitesk.requality.tests.merge.MergeProcessorAccessor;
import com.unitesk.requality.tests.merge.TestDataHelper;
import com.unitesk.requality.tests.merge.TestDataHelper.Pair;

import junit.framework.*;

public class MergeTest extends TestCase {

    private MergeProcessor mp;
    private MergeProcessorAccessor mpa;
    private Map<String, Pair<Document, Document>> testData;

    public void setUp() {
	testData = TestDataHelper.getInstance().getSanityData();
	mp = MergeProcessor.getInstance();
	mpa = new MergeProcessorAccessor(mp);
    }

    public void testMerge() {
	Document v1 = testData.get("Sanity").getKey();
	Document v2 = testData.get("Sanity").getValue();
	Document mergeResult = mp.merge(v1, v2);
	Tree t1 = new Tree(v1);
	Tree result = new Tree(mergeResult);
	assertTrue("Trees aren't equal", t1.isEqual(result));
    }

    public void testLookupLocations() {
	try {
	    Document v1 = testData.get("Sanity").getKey();
	    Tree t = new Tree(v1);
	    List<?> locations = mpa.lookupLocations(t);
	    assertEquals(4, locations.size());
	} catch (InvocationTargetException e) {
	    e.getCause().printStackTrace();
	    fail("Test failed: invoked method has throwed out exception");
	} catch (ClassCastException e) {
	    fail("Broken test case: unexpected type of returned object");
	}
    }

    public void testTraceLocation() {
	try {
	    Document v1 = testData.get("Sanity").getKey();
	    Document v2 = testData.get("Sanity").getValue();
	    Tree t1 = new Tree(v1);
	    Tree t2 = new Tree(v2);
	    List<Location> locations = mpa.lookupLocations(t1);
	    Node result = mpa.traceLocation(locations.get(0), t2);
	    assertFalse("Result is null", result == null);
	    assertTrue(
		    "Target and traced nodes aren't equal",
		    result.isEqual(locations.get(0).getSpanNode().getChilds()
			    .get(0)));
	} catch (InvocationTargetException e) {
	    e.getCause().printStackTrace();
	    fail("Test failed: invoked method has throwed out exception");
	} catch (ClassCastException e) {
	    fail("Broken test case: unexpected type of returned object");
	}
    }

    public void testTraceLocations() {
	try {
	    Document v1 = testData.get("Sanity").getKey();
	    Document v2 = v1;
	    Tree t1 = new Tree(v1);
	    Tree t2 = new Tree(v2);
	    List<Location> locations = mpa.lookupLocations(t1);
	    List<Location> result = mpa.traceLocations(locations, t2);
	    assertEquals(locations.size(), result.size());
	    for (int i = 0; i < locations.size(); i++) {
		Location loc1 = locations.get(i);
		Location loc2 = result.get(i);
		assertFalse(loc1.getAnchorNode()==null);
		assertFalse(loc2.getAnchorNode()==null);
		assertTrue(loc1.getAnchorNode().getDomNode()
			.isSameNode(loc2.getAnchorNode().getDomNode()));
	    }
	} catch (InvocationTargetException e) {
	    e.getCause().printStackTrace();
	    fail("Test failed: invoked method has throwed out exception");
	} catch (ClassCastException e) {
	    fail("Broken test case: unexpected type of returned object");
	}
    }

    public void testMapLocations() {
	try {
	    Document doc = testData.get("Sanity").getKey();
	    Tree t = new Tree(doc);
	    List<Location> locations = mpa.lookupLocations(t);
	    Map<Node, List<Location>> mapping = mpa.mapLocations(locations);
	    Iterator<Node> i = mapping.keySet().iterator();
	    while (i.hasNext()) {
		Node anchor = i.next();
		for (Location loc : mapping.get(anchor))
		    assertTrue(loc.getAnchorNode().getDomNode()
			    .isSameNode(anchor.getDomNode()));
	    }
	} catch (InvocationTargetException e) {
	    e.getCause().printStackTrace();
	    fail("Test failed: invoked method has throwed out exception");
	} catch (ClassCastException e) {
	    fail("Broken test case: unexpected type of returned object");
	}
    }
}
