package com.unitesk.requality.tests.merge.sanity;

import java.util.Iterator;
import java.util.Map;

import com.unitesk.requality.merge.Node;
import com.unitesk.requality.merge.Tree;
import com.unitesk.requality.tests.merge.TestDataHelper;
import com.unitesk.requality.tests.merge.TestDataHelper.Pair;

import org.w3c.dom.Document;

import junit.framework.*;

public class NodeTest extends TestCase {

    private Map<String, Pair<Document, Document>> testData;

    public void setUp() {
	testData = TestDataHelper.getInstance().getSanityData();
    }

    public void testIsEqual() {
	Document doc = testData.get("Sanity").getKey();
	Tree t1 = new Tree(doc);
	Tree t2 = new Tree(doc.getDocumentElement());
	Iterator<Node> i1 = t1.traverse();
	Iterator<Node> i2 = t2.traverse();
	while (i1.hasNext() && i2.hasNext()) {
	    Node x = i1.next();
	    Node y = i2.next();
	    assertTrue(x.isEqual(y));
	}
    }

    public void testIsRelated() {
	Document doc = testData.get("Sanity").getKey();
	Tree t = new Tree(doc);
	Iterator<Node> i = t.traverse();
	Node root = i.next();
	Node prev = root;
	while (i.hasNext()) {
	    Node curr = i.next();
	    assertTrue(curr.isRelated(prev) || curr.isRelated(root)
		    || !curr.getContext().isWithin(prev.getContext()));
	    prev = curr;
	}
    }
    
    public void testEquals() {
	Document doc = testData.get("Sanity").getKey();
	Tree t1 = new Tree(doc);
	Tree t2 = new Tree(doc.getDocumentElement());
	Iterator<Node> i1 = t1.traverse();
	Iterator<Node> i2 = t2.traverse();
	while (i1.hasNext() && i2.hasNext()) {
	    Node x = i1.next();
	    Node y = i2.next();
	    assertFalse(x.equals(y));
	}
    }
}
