package com.unitesk.requality.tests.merge.sanity;

import java.util.Iterator;
import java.util.Map;

import com.unitesk.requality.merge.Node;
import com.unitesk.requality.merge.NodeFilter;
import com.unitesk.requality.merge.Tree;
import com.unitesk.requality.tests.merge.TestDataHelper;
import com.unitesk.requality.tests.merge.Utils;
import com.unitesk.requality.tests.merge.TestDataHelper.Pair;

import org.w3c.dom.Document;

import junit.framework.*;

public class TreeTest extends TestCase {

    private Map<String, Pair<Document, Document>> testData;

    public void setUp() {
	testData = TestDataHelper.getInstance().getSanityData();
    }

    public void testTraverse() {
	Document doc = testData.get("Sanity").getKey();
	Tree t = new Tree(doc);
	Iterator<Node> i = t.traverse();
	Node prev = i.next();
	while (i.hasNext()) {
	    Node curr = i.next();
	    assertFalse(Utils.isParent(curr, prev));
	    prev = curr;
	}
    }

    public void testNearestMatch() {
	Document doc = testData.get("Sanity").getKey();
	Tree t = new Tree(doc);
	for (Node x : t.getTraversalSequence()) {
	    Node nearest1 = t.nearestMatch(x);
	    Node nearest2 = x;
	    Node nearest3 = t.nearestMatch(x, new NodeFilter(x, true, true,
		    true, false));
	    Node nearest4 = x;
	    if (x.getParent() != null) {
		nearest2 = t.nearestMatch(x, x.getParent());
		nearest4 = t.nearestMatch(x, x.getParent(), new NodeFilter(x,
			true, true, true, false));
	    }
	    assertFalse("Result is null", nearest1 == null || nearest2 == null
		    || nearest3 == null || nearest4 == null);
	    assertTrue(x.equals(nearest1) && x.equals(nearest2)
		    && x.equals(nearest3) && x.equals(nearest4));
	}
    }
}
